%% Example_IODMD
%
% This m-file uses the IODMD method to construct a reduced order model
% for the nonlinear mass-spring-damper system. The dynamics for the
% nonlinear MSD system are described further in NonlinearMSD.m.

%% System parameters
M = 100;        % # of masses
m = 1;          % Mass of individual blocks, kg
k1 = 0.5;       % Linear spring constant, N/m
k2 = 1;         % Cubic spring constant, N/m^3
b1 = 1;         % Linear damping constant, N/(m/s)
b2 = 0;         % Cubic damping constant, N/(m/s)^3

%% Equilibrium condition

% Parameter specifying equilibrium point, F=rho (in N)
%rho = 0;
rho = 1;
%rho = 10;

% Simulate nonlinear system to find trim condition
xbar = ComputeMSDTrim(rho,M,m,k1,k2,b1,b2);

%% Compute snapshots from nonlinear system

% Simulation times
dt = 0.1;          % Time step, sec
Tf = 20;           % Final time, sec
tin = 0:dt:Tf;     % Time vector for snapshots, sec

% Chirp excitation for input force u, N
if rho==0
    w0 = 0.1; wf = 1;
else
    w0 = 0.1; wf = 2;
end
ain = 1e-1;
win = w0*(wf/w0).^(tin/Tf);
uin = ain*sin( win.*tin );
u = [tin(:) uin(:)];

% Constant (trim) for exogenous force F, N
F = [0 rho; Tf rho];

% Function handle to compute the system derivative
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Initial Conditions: Use trim condition computed above
x0 = xbar;

% Simulate Nonlinear System
opt = odeset('AbsTol',1e-6,'RelTol',1e-6);
[tnl,xnl]=ode45(xdotfh,tin,x0,opt);

%% Input/Output DMD method for low-order model construction

% Collect state, input, output snapshots (removing trim condition)
Nt = numel(tnl);
X = xnl'-repmat(x0,[1 Nt]);
U = uin;
Y = X(M,:);

% Compute reduced order models for a variety of dimensions
Nqvec = 1:10;
[Gredvec,Qvec,Epodrel,Efitrel]=IODMD(X,U,Y,dt,Nqvec);

% Plot relative POD and model fit errors
figure(1);
subplot(211)
semilogy(Nqvec,Epodrel,'bs');
grid on;
ylabel('POD Rel Error');

subplot(212)
semilogy(Nqvec,Efitrel,'bs');
grid on; set(gca,'YMinorGrid','off')
ylabel('Model Fit Rel Error');
xlabel('# of Modes');
if exist('garyfyFigure','file')
    garyfyFigure;
end

% Select state dimension based on fitting error
Nq = 4;
Q = Qvec(:,1:Nq);
Gred = Gredvec(:,:,Nq);

%% Compare Bode plot of reduced and full order linear systems

% True (full-order) linearized system
Glin = LinearizedMSD(xbar,m,k1,k2,b1);

% Bode Plots
figure(2);
bopt = bodeoptions;
bopt.PhaseMatching='on';
bode(Glin,'b',Gred,'r-.',bopt)
legend('Full','Reduced','Location','Best');
if exist('garyfyFigure','file')
    garyfyFigure;
end

%% Compare reduced-order linear and full nonlinear step responses

% Step for input force u, N
ustep = 1e-1;
u = [0 ustep; Tf ustep];

% Function handle to compute the system derivative
% Use constant exogeneous force F specified above.
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Simulate Nonlinear System
% Use initial condition x0 and final time Tf specified above
[tnl,xnl]=ode45(xdotfh,[0 Tf],x0);
Nt = numel(tnl);
delxnl = xnl - repmat(xbar',[Nt,1]);
delynl = delxnl(:,M);

% Simulate Linearized System
% STEP computes a unit step response. Scale output and state by ustep.
[yred,tred,zred]=step(Gred,Tf);
delyred = yred*ustep;
delzred = zred*ustep;

% Plot: Deviations of output (block M position) from trim condition
figure(3); 
plot(tnl,delynl,'b',tred,delyred,'r--')
ylim([min(delyred) 1.1*max(delyred)]);
xlabel('Time, sec');
ylabel('\delta_y, m');
grid on;
legend('Nonlin','Red','Location','Best')
if exist('garyfyFigure','file')
    garyfyFigure;
end

% Plot: Deviations of several block positions from trim
blkidx = 98:100; % Select block positions to plot
Nplot = numel(blkidx);
figure(4);
for i=1:Nplot
    % Use modes and reduced order state to estimate position
    blki = blkidx(i);
    delpred = Q(blki,:)*delzred';
    
    subplot(Nplot,1,i);
    plot(tnl,delxnl(:,blki),'b',tred,delpred,'r--');
    ylim([min(delpred) 1.1*max(delpred)]);
    ylabel(['\delta_x(' int2str(blki) ') , m']);
    grid on;
end
xlabel('Time, sec');
legend('Nonlin','Red','Location','Best')
if exist('garyfyFigure','file')
    garyfyFigure;
end
