%% Example_SingleBlockLPV
%
% This m-file studies the LPV linearization for the one-block nonlinear
% mass-spring-damper system.

%% System parameters
M = 1;          % # of masses
m = 1;          % Mass of individual blocks, kg
k1 = 0.5;       % Linear spring constant, N/m
k2 = 1;         % Cubic spring constant, N/m^3
b1 = 1;         % Linear damping constant, N/(m/s)
b2 = 0;         % Cubic damping constant, N/(m/s)^3

%% Simulations: Compare Nonlinear and "Frozen" Linear Results
% This is used to verify the discrete-time linearization.

% Trim condition for linearization
rho = 2;       % Trim force input, N
xbar = ComputeMSDTrim(rho,M,m,k1,k2,b1,b2);

% Discrete-time linearization
dt = 0.01;
Glin = LinearizedMSD(xbar,m,k1,k2,b1,dt);

% Final simulation time, sec
Tf = 20;           

% Input forces, N
ustep = 1e-2;
u = [0 ustep; Tf ustep];
F = [0 rho; Tf rho];

% Function handle to compute the system derivative
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Initial Conditions: Use trim condition computed above
x0 = xbar;

% Simulate Nonlinear System
[tnl,xnl]=ode45(xdotfh,[0 Tf],x0);
delpnl = xnl(:,1)-x0(1);
delvnl = xnl(:,2)-x0(2);

% Simulate Linearized System
% STEP computes a unit step response. Scale output and state by ustep
[ylin,tlin,xlin]=step(Glin,Tf);
delxlin = xlin*ustep;
delplin = delxlin(:,1);
delvlin = delxlin(:,2);

% Plot deviations of output from trim condition
figure(1);
subplot(2,1,1);
plot(tnl,delpnl,'b',tlin,delplin,'r-.')
ylabel('\delta_x(1), m');

subplot(2,1,2);
plot(tnl,delvnl,'b',tlin,delvlin,'r-.')
xlabel('Time, sec');
ylabel('\delta_x(2), m/s');
legend('NL','Lin','Location','Best')
if exist('garyfyFigure','file')
    garyfyFigure;
end

%% Simulations: Compare Nonlinear and Linear Parameter-Varying Results
% These simulations are performed for a time-varying rho.

% Specify sinusoidal parameter trajectory rho(t)
Tf = 100;
tin = 0:dt:Tf;
Nt=numel(tin);

amp = 0.8;
bias = 0;
freq = 0.1;
rhoin = amp*cos(freq*tin)+bias;
F = [tin(:) rhoin(:)];

% Input Force, N
u = [0 0; Tf 0];

% Function handle to compute the system derivative
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Initial State: Use trim condition associated with initial rho
x0 = ComputeMSDTrim(rhoin(1),M,m,k1,k2,b1,b2);

% Simulate Nonlinear System
[tnl,xnl]=ode45(xdotfh,tin,x0);

% Linear Simulation
delxlpv = zeros(2,Nt);
xbar = zeros(2,Nt);
xbar(:,1) = x0;
for k=1:(Nt-1)
    % Parameter value at time k
    rhok = rhoin(k);

    % Trim Condition at time k+1
    xbar(:,k+1) = ComputeMSDTrim(rhoin(k+1),M,m,k1,k2,b1,b2);
    
    % Linearized state matrix at time k, G(rho(k))
    [Ak,~,~,~] = LinearizedMSD(xbar(:,k),m,k1,k2,b1,dt);
    
    % Update State (Assumes delu =0)
    delxlpv(:,k+1) = Ak*delxlpv(:,k) + ( xbar(:,k) - xbar(:,k+1) );
end
xlpv = delxlpv+xbar;

%% Plot results vs time
figure(2)
subplot(211)
plot(tnl,xnl(:,1),'b',tin,xlpv(1,:),'r-.');
ylabel('Position, m');
grid on;

subplot(212);
plot(tnl,xnl(:,2),'b',tin,xlpv(2,:),'r-.'); 
ylabel('Velocity, m/s');
legend('Nonlin','LPV','Location','Best');
xlabel('Time, sec');
grid on;
if exist('garyfyFigure','file')
    garyfyFigure;
end


%% Plot results vs. parameter input

% Compute trim states for many values of rho
rhotrim = -1:0.05:1;       % Grid of trim force inputs, N
Ntrim = numel(rhotrim);
xtrim = zeros(2,Ntrim);
for i=1:Ntrim
    xtrim(:,i) = ComputeMSDTrim(rhotrim(i),M,m,k1,k2,b1,b2);
end

% Plot results vs parameter input
figure(3);
subplot(211)
ph1=plot(rhoin,xnl(:,1),'b',rhoin,xlpv(1,:),'r-.',...
    rhotrim,xtrim(1,:),'g-o');
ylabel('Position, m');
grid on;
legend('Nonlin','LPV','Trim');

subplot(212);
plot(rhoin,xnl(:,2),'b',rhoin,xlpv(2,:),'r-.');
hold on;
ph2=plot(rhotrim,xtrim(2,:),'g-o');
hold off;
xlabel('Force \rho:=F, N')
ylabel('Velocity, m/s');
grid on;
if exist('garyfyFigure','file')
    garyfyFigure;
end
set([ph1(3); ph2],'LineWidth',1.5,'MarkerSize',3);

